package cn.ibizlab.core.sample.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.core.sample.domain.IBIZTASK;
import cn.ibizlab.core.sample.filter.IBIZTASKSearchContext;
import cn.ibizlab.core.sample.service.IIBIZTASKService;

import cn.ibizlab.util.helper.CachedBeanCopier;
import cn.ibizlab.util.helper.DEFieldCacheMap;
import com.alibaba.fastjson.JSONObject;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.core.sample.mapper.IBIZTASKMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import org.springframework.util.StringUtils;

/**
 * 实体[任务] 服务对象接口实现
 */
@Slf4j
@Service("IBIZTASKServiceImpl")
public class IBIZTASKServiceImpl extends ServiceImpl<IBIZTASKMapper, IBIZTASK> implements IIBIZTASKService {

    @Autowired
    @Lazy
    protected cn.ibizlab.core.sample.service.IIBIZTASKTEAMService ibiztaskteamService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(IBIZTASK et) {
        if(!this.retBool(this.baseMapper.insert(et))) {
            return false;
        }
        ibiztaskteamService.saveByRoot(et.getIbiztaskid(), et.getIbiztaskteam());
        CachedBeanCopier.copy(get(et.getIbiztaskid()), et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<IBIZTASK> list) {
        this.saveBatch(list, batchSize);
    }

    @Override
    @Transactional
    public boolean update(IBIZTASK et) {
        if(!update(et, (Wrapper) et.getUpdateWrapper(true).eq("ibiztaskid", et.getIbiztaskid()))) {
            return false;
        }
        ibiztaskteamService.saveByRoot(et.getIbiztaskid(), et.getIbiztaskteam());
        CachedBeanCopier.copy(get(et.getIbiztaskid()), et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<IBIZTASK> list) {
        updateBatchById(list, batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        ibiztaskteamService.removeByRoot(key) ;
        boolean result = removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public IBIZTASK get(String key) {
        IBIZTASK et = getById(key);
        if (et == null) {
            throw new BadRequestAlertException("数据不存在", this.getClass().getSimpleName(), key);
        }
        else {
            et.setIbiztaskteam(ibiztaskteamService.selectByRoot(key));
        }
        return et;
    }

    @Override
    public IBIZTASK getDraft(IBIZTASK et) {
        return et;
    }

    @Override
    public boolean checkKey(IBIZTASK et) {
        return (!ObjectUtils.isEmpty(et.getIbiztaskid())) && (!Objects.isNull(this.getById(et.getIbiztaskid())));
    }
    @Override
    @Transactional
    public boolean save(IBIZTASK et) {
        if(!saveOrUpdate(et)) {
            return false;
        }
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(IBIZTASK et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? getProxyService().update(et) : getProxyService().create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<IBIZTASK> list) {
        List<IBIZTASK> create = new ArrayList<>();
        List<IBIZTASK> update = new ArrayList<>();
        for (IBIZTASK et : list) {
            if (ObjectUtils.isEmpty(et.getIbiztaskid()) || ObjectUtils.isEmpty(getById(et.getIbiztaskid()))) {
                create.add(et);
            } else {
                update.add(et);
            }
        }
        if (create.size() > 0) {
            getProxyService().createBatch(create);
        }
        if (update.size() > 0) {
            getProxyService().updateBatch(update);
        }
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<IBIZTASK> list) {
        List<IBIZTASK> create = new ArrayList<>();
        List<IBIZTASK> update = new ArrayList<>();
        for (IBIZTASK et : list) {
            if (ObjectUtils.isEmpty(et.getIbiztaskid()) || ObjectUtils.isEmpty(getById(et.getIbiztaskid()))) {
                create.add(et);
            } else {
                update.add(et);
            }
        }
        if (create.size() > 0) {
            getProxyService().createBatch(create);
        }
        if (update.size() > 0) {
            getProxyService().updateBatch(update);
        }
    }



    /**
     * 查询集合 数据集
     */
    @Override
    public Page<IBIZTASK> searchDefault(IBIZTASKSearchContext context) {
            com.baomidou.mybatisplus.extension.plugins.pagination.Page<IBIZTASK> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
            return new PageImpl<IBIZTASK>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<IBIZTASK> getIbiztaskByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<IBIZTASK> getIbiztaskByEntities(List<IBIZTASK> entities) {
        List ids =new ArrayList();
        for(IBIZTASK entity : entities){
            Serializable id=entity.getIbiztaskid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0) {
            return this.listByIds(ids);
        }
        else {
            return entities;
        }
    }




    public IIBIZTASKService getProxyService() {
        return cn.ibizlab.util.security.SpringContextHolder.getBean(this.getClass());
    }
}



